package com.kakao.sdk.auth.network

import com.kakao.sdk.auth.AccessTokenRepo
import com.kakao.sdk.common.Constants
import okhttp3.Interceptor
import okhttp3.Request
import okhttp3.Response

/**
 * API 요청에 AccessToken을 추가하는 인터셉터
 *
 * TODO: Authenticator로 구현해보자
 */
class AccessTokenInterceptor(
    private val tokenRepo: AccessTokenRepo = AccessTokenRepo.instance
) : Interceptor {
    override fun intercept(chain: Interceptor.Chain): Response {
        val request =
            tokenRepo.getToken().accessToken?.let {
                requestWithAuthorization(chain.request(), it)
            } ?: chain.request()

        return chain.proceed(request)
    }

    companion object {
        fun requestWithAuthorization(request: Request, accessToken: String): Request =
            request.newBuilder()
                .removeHeader(Constants.AUTHORIZATION)
                .addHeader(Constants.AUTHORIZATION, "${Constants.BEARER} $accessToken")
                .build()
    }
}
