package com.kakao.sdk.link

import com.kakao.sdk.link.model.ImageUploadResult
import com.kakao.sdk.link.model.ValidationResult
import com.kakao.sdk.template.DefaultTemplate
import okhttp3.MultipartBody
import retrofit2.Call
import retrofit2.http.*

interface LinkApi {
    @GET("${Constants.VALIDATE_CUSTOM_PATH}?link_ver=4.0")
    fun validateCustom(
        @Query(Constants.TEMPLATE_ID) templateId: Long,
        @Query(Constants.TEMPLATE_ARGS) templateArgs: Map<String, String>? = null
    ): Call<ValidationResult>

    @GET("${Constants.VALIDATE_DEFAULT_PATH}?link_ver=4.0")
    fun validateDefault(@Query(Constants.TEMPLATE_OBJECT) templateObject: DefaultTemplate): Call<ValidationResult>

    @GET("${Constants.VALIDATE_SCRAP_PATH}?link_ver=4.0")
    fun validateScrap(
        @Query(Constants.REQUEST_URL) url: String,
        @Query(Constants.TEMPLATE_ID) templateId: Long? = null,
        @Query(Constants.TEMPLATE_ARGS) templateArgs: Map<String, String>? = null
    ): Call<ValidationResult>

    @Multipart
    @POST(Constants.UPLOAD_IMAGE_PATH)
    fun uploadImage(
        @Part image: MultipartBody.Part,
        @Part("secure_resource") secureResource: Boolean? = true
    ): Call<ImageUploadResult>


    @POST(Constants.SCRAP_IMAGE_PATH)
    @FormUrlEncoded
    fun scrapImage(
        @Field("image_url") imageUrl: String,
        @Field("secure_resource") secureResource: Boolean? = true
    ): Call<ImageUploadResult>
}