package com.kakao.sdk.network

import com.kakao.sdk.common.KakaoJson
import com.kakao.sdk.common.model.ApiError
import com.kakao.sdk.common.model.ApiErrorCause
import com.kakao.sdk.common.model.ApiErrorResponse
import com.kakao.sdk.common.model.KakaoSdkError
import retrofit2.Call
import retrofit2.Callback
import retrofit2.HttpException
import retrofit2.Response

abstract class ApiCallback<T> : Callback<T> {
    abstract fun onComplete(model: T?, error: Throwable?)

    override fun onFailure(call: Call<T>, t: Throwable) {
        if (t.cause is KakaoSdkError) {
            onComplete(null, t.cause)
        } else {
            onComplete(null, t)
        }
    }

    override fun onResponse(call: Call<T>, response: Response<T>) {
        val model = response.body()
        if (model != null) {
            onComplete(model, null)
        } else {
            onFailure(call, translateError(HttpException(response)))
        }
    }

    companion object {
        fun translateError(t: Throwable): Throwable {
            try {
                if (t is HttpException) {
                    val errorString = t.response()?.errorBody()?.string()
                    val response =
                        KakaoJson.fromJson<ApiErrorResponse>(
                            errorString!!,
                            ApiErrorResponse::class.java
                        )
                    val cause =
                        KakaoJson.fromJson(
                            response.code.toString(),
                            ApiErrorCause::class.java
                        )
                            ?: ApiErrorCause.Unknown
                    return ApiError(t.code(), cause, response)
                }
                return t
            } catch (unexpected: Throwable) {
                return unexpected
            }
        }
    }
}
