/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.user

import com.kakao.sdk.auth.network.kapiWithOAuth
import com.kakao.sdk.network.ApiCallback
import com.kakao.sdk.network.ApiFactory
import com.kakao.sdk.user.model.AccessTokenInfo
import com.kakao.sdk.user.model.User
import com.kakao.sdk.user.model.UserServiceTerms
import com.kakao.sdk.user.model.UserShippingAddresses
import java.util.*

/**
 * @author kevin.kang. Created on 2019-11-01..
 *
 * @suppress
 */
class UserApiClient (
    private val userApi: UserApi = ApiFactory.kapiWithOAuth.create(UserApi::class.java)
) {

    fun me(secureReSource: Boolean = true, callback: (user: User?, error: Throwable?) -> Unit) {
        userApi.me(secureReSource)
            .enqueue(object: ApiCallback<User>() {
                override fun onComplete(model: User?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun accessTokenInfo(callback: (info: AccessTokenInfo?, error: Throwable?) -> Unit) {
        userApi.accessTokenInfo()
            .enqueue(object: ApiCallback<AccessTokenInfo>() {
                override fun onComplete(model: AccessTokenInfo?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun updateProfile(properties: Map<String, String>, callback: (error: Throwable?) -> Unit) {
        userApi.updateProfile(properties)
            .enqueue(object: ApiCallback<Unit>() {
                override fun onComplete(model: Unit?, error: Throwable?) {
                    callback(error)
                }
            })
    }

    fun logout(callback: (error: Throwable?) -> Unit) {
        userApi.logout()
            .enqueue(object: ApiCallback<Unit>() {
                override fun onComplete(model: Unit?, error: Throwable?) {
                    callback(error)
                }
            })
    }

    fun unlink(callback: (error: Throwable?) -> Unit) {
        userApi.unlink()
            .enqueue(object: ApiCallback<Unit>() {
                override fun onComplete(model: Unit?, error: Throwable?) {
                    callback(error)
                }
            })
    }

    fun shippingAddresses(
        addressId: Long? = null,
        fromUpdateAt: Date? = null,
        pageSize: Int? = null,
        callback: (userShippingAddresses: UserShippingAddresses?, error: Throwable?) -> Unit
    ) {
        userApi.shippingAddresses(addressId, fromUpdateAt, pageSize)
            .enqueue(object: ApiCallback<UserShippingAddresses>() {
                override fun onComplete(model: UserShippingAddresses?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    fun serviceTerms(callback: (userServiceTerms: UserServiceTerms?, error: Throwable?) -> Unit) {
        userApi.serviceTerms()
            .enqueue(object: ApiCallback<UserServiceTerms>() {
                override fun onComplete(model: UserServiceTerms?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    companion object {
        val instance by lazy { UserApiClient() }
    }
}
